/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://kenai.com/hg/openbluesky~src
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy.spi.simple;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.hierarchy.HItem;
import it.tidalwave.hierarchy.HView;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class SimpleHItem extends ParentSupport<HItem> implements HItem
  {
    @Nonnull
    private final HView view;

    @CheckForNull
    private final HItem parent;

    public SimpleHItem (final @Nonnull SimpleBuilder<HItem> builder,
                                 final @Nonnull HView view,
                                 final @Nullable HItem parent)
      {
        super(builder, "SimpleHItem", (parent != null) ? parent : view);
        this.view = view;
        this.parent = parent;
      }

    @Override @Nonnull
    public HView getView()
      {
        return view;
      }

    @Override @Nonnull
    public HItem getParent()
      throws NotFoundException
      {
        return NotFoundException.throwWhenNull(parent, "No parent for " + this);
      }

    @Override @Nonnull
    protected HItem createItem (final @Nonnull SimpleBuilder<HItem> builder)
      {
        return new SimpleHItem(builder, view, this);
      }
  }
