/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://kenai.com/hg/openbluesky~src
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy.spi.simple;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;
import it.tidalwave.util.Id;
import it.tidalwave.role.spi.DefaultDisplayable;
import it.tidalwave.netbeans.util.test.Resettable;
import it.tidalwave.netbeans.capabilitiesprovider.LookupFactory;
import it.tidalwave.hierarchy.Builder;
import it.tidalwave.hierarchy.Finder;
import it.tidalwave.hierarchy.HView;
import it.tidalwave.hierarchy.HViewManager;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class SimpleHViewManager extends AsSupport implements HViewManager
  {
    private final List<HView> views = new ArrayList<HView>();
    private final Map<Id, SimpleHView> viewMapById = new HashMap<Id, SimpleHView>();

    @CheckForNull
    private Lookup lookup;

    private final Resettable resetter = new Resettable()
      {
        @Override
        public void reset()
          {
            views.clear();
            viewMapById.clear();
          }
      };

    @Override @Nonnull
    public Builder<HView> createView()
      {
        return new SimpleBuilder<HView>()
          {
            @Override @Nonnull
            public HView build()
              {
                return createView(this);
              }
          };
      }

    @Override @Nonnull
    public Builder<HView> findOrCreateView()
      {
        return new SimpleBuilder<HView>()
          {
            @Override @Nonnull
            public HView build()
              {
                if (viewMapById.containsKey(id))
                  {
                    return viewMapById.get(id);
                  }

                return createView(this);
              }
          };
      }

    @Override @Nonnull
    public Finder<HView> findViews()
      {
        return new HFinderSupport<HView>("findViews()")
          {
            @Override @Nonnull
            protected List<HView> computeResults()
              {
                if (id != null)
                  {
                    final SimpleHView view = viewMapById.get(id);
                    return (view != null) ? Collections.<HView>singletonList(view) : Collections.<HView>emptyList();
                  }
                else
                  {
                    return new ArrayList<HView>(views);
                  }
              }      
          };
      }

    @Override @Nonnull
    public Finder<HView> findItems()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Nonnull
    private HView createView (final @Nonnull SimpleBuilder<HView> builder)
      {
        final SimpleHView view = new SimpleHView(builder, this);
        views.add(view);
        viewMapById.put(view.getId(), view);
        return view;
      }

    @Override @Nonnull
    public synchronized Lookup getLookup()
      {
        if (lookup == null)
          {
            final String displayName = NbBundle.getMessage(SimpleHViewManager.class, "HViewManager.displayName");
            final DefaultDisplayable displayable = new DefaultDisplayable(displayName, "SimpleHViewManager");
            lookup = LookupFactory.createLookup(this, Lookups.fixed(displayable, resetter));
          }

        return lookup;
      }
  }
